/*
  myTemp is used to connect PalmOS PDAs to Dallas Microlan(tm) Devices
  It currently supports DS1820/DS18S20 Digital Thermometers
  Copyright (C) 2001 Christof Klaiber

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA  02111-1307  USA


  Christof Klaiber <christof.klaiber@gmx.de>
  Lachnerstr. 22
  76131 Karlsruhe
  Germany
*/

/*
	PalmOS Headers
*/

#include <PalmOS.h>
#include <SerialMgrOld.h>

/*
	myTemp Headers
*/

#include "TempRsc.h"
#include "Temp.h"
#include "ds1820.h"
#include "onewire.h"

/*
	Defines for the Temperature Database
*/

#define kTempDBType	'Temp'
#define kTempDBName	"myTemp-Temp"

/*
	Read Timeout
*/
const	UInt16 gTimeout = 5 * sysTicksPerSecond;

/*
	Wait between Conversion and Read command
*/
const	UInt16 gReadTime = sysTicksPerSecond * 0.75;

/*
	Wait one second to power up the serial line
*/
const	UInt16 gVoltageTime = sysTicksPerSecond;

/*
	Sample Period Times
*/
const	UInt16 gSamplePeriods[] = {0, 1, 2, 5, 10, 15, 20, 30, 45, 60, 90, 120, 180, 300, 600};


/*
	Too make life easier, we support only up to ten sensors
*/
Char*	gSensors[] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10"};


UInt32	gNextSample;
MemHandle	gHandles[3][kNumRows];
UInt16	gOffset = 0;
Char	gSampleLabel[10];
Char	gTempString[8];
Char	gMaxString[8];
Char	gMinString[8];
UInt8	gSensor = 0;

DmOpenRef	gTempDB;	

struct myAppPrefs {
	Int16 MaxTemp;
	Int16 MinTemp;
	Boolean	Metric;
	UInt8 SamplePeriod;
	Boolean Log;
	Boolean	LogAll;
};

struct mySampleType {
	UInt32	Ticks;
	Int16	Temp;
	UInt8	Sensor;
};

struct myAppPrefs	gAppPrefs;

Int16 do_temp(UInt8 *scratch) {
	UInt16	hi_precision;
	Int16	temp;
	
	if (scratch[1] == 0)
	{
		temp = scratch[0] >> 1;
	} else {
		temp = -1 * (0x100 - scratch[0]) >> 1;
	}
	
	temp *= 100;
	
	temp -= 25;
	
	hi_precision = scratch[7] - scratch[6];
	
	hi_precision *= 100;
	
	if (scratch[7] != 0)
		hi_precision /= scratch[7];
	else 
		return 13000;
	
	temp += hi_precision;
#ifdef DS1820
	temp += 320;
#endif
	return temp;
}

static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
        UInt32  romVersion;
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
        if (romVersion < requiredVersion)
        {
                if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) == (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
                        FrmAlert(RomIncompatibleAlert);
                        if (romVersion < 0x02000000) {
                                AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
                        }
                }
                return sysErrRomIncompatible;
        }
}


static Boolean StartApplication(launchFlags) {
	Err	err;
	UInt16	i;
	UInt16	j;
	UInt16	prefSize;
	UInt16	currentPosition;
	UInt16	textHeight;
	UInt16	maxValue;
	Char*	s;
	
	gSerialRefNum = NULL;
	
	err = RomVersionCompatible(0x3003000, launchFlags);
	if (err)
		return err;
	
	prefSize = sizeof(gAppPrefs);
	if(PrefGetAppPreferences(kTempID, 0, &gAppPrefs, &prefSize, true) == noPreferenceFound) {
		gAppPrefs.MaxTemp = -6000;
		gAppPrefs.MinTemp = 12500;
		gAppPrefs.Metric = true;
		gAppPrefs.SamplePeriod = 4;
		gAppPrefs.Log = false;
		gAppPrefs.LogAll = false;
	}
		
	gTempDB = DmOpenDatabaseByTypeCreator(kTempDBType, kTempID, dmModeReadWrite);
	err = DmGetLastErr();
	if (! gTempDB)
	{
		err = DmCreateDatabase(0, kTempDBName, kTempID, kTempDBType, false);
		if (err)
			return err;
		gTempDB = DmOpenDatabaseByTypeCreator(kTempDBType, kTempID, dmModeReadWrite);
		if (!gTempDB)
			return DmGetLastErr();
	}
	
	for (i = 0; i < kNumRows; i++)
		for(j = 0; j < 3; j++) {
			gHandles[j][i] = MemHandleNew(1);
			s = MemHandleLock(gHandles[j][i]);
			*s = '\0';
			MemHandleUnlock(gHandles[j][i]);
		}
	
	
	return false;
}

static void StopApplication(void) {
	Err	err;
	UInt16	i;
	UInt16	j;
	
	if(gSerialRefNum) {
		SerSetReceiveBuffer(gSerialRefNum, NULL, 0);
		SerClose(gSerialRefNum);
		gSerialRefNum = NULL;
	}
	PrefSetAppPreferences(kTempID, 0, kTempVer, &gAppPrefs, sizeof(gAppPrefs), true);
	
	for (i = 0; i < kNumRows; i++)
		for(j = 0; j < 3; j++) {
			MemHandleFree(gHandles[j][i]);
		}
	
#ifndef DEMO	
	if (rom_list.roms != NULL)
		MemHandleFree(rom_list.roms);
#endif		
	err = DmCloseDatabase(gTempDB);

	FrmCloseAllForms();
}

static Int32 c2f(Int32 temp) {
	temp *=9;
	temp /= 5;
	temp += 3200;
	return temp;
}

static void IntToTemp(Int16 temp, Char* tempPtr, Char* sep) {
	Int16	temp_c;
	Int16	temp_d;
	UInt16	objectIndex;

	Char	resultPtr[10];
	
	if ((temp <= -6000) || (temp >= 12500)) {
		tempPtr = "x.xx";
	} else {
		
		if (!gAppPrefs.Metric) {
			temp = c2f(temp);
		}

		temp_c = temp / 100;
		temp_d = abs(temp % 100);

		StrIToA(tempPtr, temp_c);
		StrCat(tempPtr, sep);
		StrIToA(resultPtr, temp_d);
		if (StrLen(resultPtr) < 2) {
			resultPtr[2] = 0;
			resultPtr[1] = resultPtr[0];
			resultPtr[0] = '0';
		}

		StrCat(tempPtr, resultPtr);
	}
}

static UInt8 WhichTextColumn(int column) {
	if (column == 1)
		return 0;
	else if (column == 2)
		return 1;
	else if (column == 3)
		return 2;
	else 
		return 3;
	
}

static void SecondsToString(UInt32 seconds, Char* date){
	Char	tempPtr[9];
	DateTimeType	datetime;

	TimSecondsToDateTime(seconds, &datetime);
			
			
	StrCopy(date, "");
	StrIToA(tempPtr, datetime.hour);
	if (StrLen(tempPtr) < 2) {
		tempPtr[2] = tempPtr[1];
		tempPtr[1] = tempPtr[0];
		tempPtr[0] = '0';
	}
	StrCat(date, tempPtr);
	StrCat(date, ":");
	StrIToA(tempPtr, datetime.minute);
	if (StrLen(tempPtr) < 2) {
		tempPtr[2] = tempPtr[1];
		tempPtr[1] = tempPtr[0];
		tempPtr[0] = '0';
	}
	StrCat(date, tempPtr);
	StrCat(date, ":");
	StrIToA(tempPtr, datetime.second);
	if (StrLen(tempPtr) < 2) {
		tempPtr[2] = tempPtr[1];
		tempPtr[1] = tempPtr[0];
		tempPtr[0] = '0';
	}
	StrCat(date, tempPtr);
}

static Err CustomLoadTemp(void* table, Int16 row, Int16 column, Boolean editable, MemHandle* dataH, Int16* dataOffset, Int16* dataSize, FieldPtr fld) {
	Err	err;
	UInt16	numRecords = DmNumRecords(gTempDB);
	MemPtr	myRecord;
	Char	tempPtr[12];
	Char*	tempNew;
	struct	mySampleType *samplePtr;
	
	if (numRecords > row) {
		myRecord = DmQueryRecord(gTempDB, row + gOffset);
		samplePtr = MemHandleLock(myRecord);
		if (column == 3) {
			IntToTemp(samplePtr->Temp, tempPtr, ",");

			err = MemHandleResize(gHandles[WhichTextColumn(column)][row], StrLen(tempPtr) + 1);

			tempNew = MemHandleLock(gHandles[WhichTextColumn(column)][row]);
			StrCopy(tempNew, tempPtr);
			MemHandleUnlock(gHandles[WhichTextColumn(column)][row]);
		}  else if (column == 1) {
			SecondsToString(samplePtr->Ticks, tempPtr);
			err = MemHandleResize(gHandles[WhichTextColumn(column)][row], StrLen(tempPtr) + 1);
			tempNew = MemHandleLock(gHandles[WhichTextColumn(column)][row]);
			StrCopy(tempNew, tempPtr);
			MemHandleUnlock(gHandles[WhichTextColumn(column)][row]);
		} else if (column == 2) {
			StrIToA(tempPtr, samplePtr->Sensor + 1);
			
			err = MemHandleResize(gHandles[WhichTextColumn(column)][row], StrLen(tempPtr) + 1);
			tempNew = MemHandleLock(gHandles[WhichTextColumn(column)][row]);
			StrCopy(tempNew, tempPtr);
			MemHandleUnlock(gHandles[WhichTextColumn(column)][row]);
		}
		MemHandleUnlock(myRecord);
	} else {
		MemHandleResize(gHandles[WhichTextColumn(column)][row], 1);
		tempNew = MemHandleLock(gHandles[WhichTextColumn(column)][row]);
		*tempNew = '\0';
		MemHandleUnlock(gHandles[WhichTextColumn(column)][row]);
	}

	*dataH = gHandles[WhichTextColumn(column)][row];
	*dataOffset = 0;
	*dataSize = MemHandleSize(*dataH);
	return 0;
}

static void UpdateScrollbar(void) {
	FormPtr		frm = FrmGetActiveForm();
	ScrollBarPtr	scroll;
	UInt16		textHeight;
	UInt16		maxValue;
	
	
	textHeight = DmNumRecords(gTempDB);

	if (textHeight > kNumRows)
		maxValue = textHeight - kNumRows;
	else
		maxValue = 0;

	scroll = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempScrollBar));
		
	SclSetScrollBar(scroll, gOffset, 0, maxValue, kNumRows - 1);
}

static UInt16 GetRowNum(UInt16 row) {
	Int16	result;
	
	result =  DmNumRecords(gTempDB) - (row + gOffset);
	
	if (result < 0)
		return 0;
	else
		return result % 1000;
}

static Boolean OpenSerial(FormPtr frm) {
	Err	err;
	Boolean	res = true;
	SerSettingsType term;
	ControlPtr	ctl;

	SysCopyStringResource(gSampleLabel, WaitString);
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SampleButton));
	CtlSetLabel(ctl, gSampleLabel);

#ifndef DEMO
	err = SysLibFind("Serial Library", &gSerialRefNum);
	if(err == sysErrLibNotFound )
	{
		err = SysLibLoad('libr', 'swrp', &gSerialRefNum);
		ErrNonFatalDisplayIf(err, "can't load serial library");
	}
	ErrNonFatalDisplayIf(err != 0, "can't find serial library.");
	
	err = SerOpen(gSerialRefNum, 0, 115200);
	if (err != 0) {
		if (err == serErrAlreadyOpen) {
			FrmAlert(SerialInUseAlert);
			SrmClose(gSerialRefNum);
			res = false;
		} else {
			FrmAlert(CantopenserialAlert);
			res = false;
		}
		return res;
	}

#endif

	SysTaskDelay(gVoltageTime);

#ifndef DEMO

	err = SerSetReceiveBuffer(gSerialRefNum, gSerialBuffer, sizeof(gSerialBuffer));
	
	term.baudRate = 115200;
	term.flags = serDefaultSettings;
	err = SerSetSettings(gSerialRefNum, &term);

#endif
		
	SysCopyStringResource(gSampleLabel, SampleString);
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SampleButton));
	CtlSetLabel(ctl, gSampleLabel);
	
	return res;
}

static Boolean CloseSerial(FormPtr frm) {
	ControlPtr	ctl;

#ifndef DEMO
	SerSetReceiveBuffer(gSerialRefNum, NULL, 0);
	SerClose(gSerialRefNum);
	
	gSerialRefNum = NULL;
#endif
		
	SysCopyStringResource(gSampleLabel, IdleString);	
		
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SampleButton));
	
	CtlSetLabel(ctl, gSampleLabel);

	return true;
}



static Int8 GetROM(FormPtr frm) {
	Int8	res;
	ControlPtr	ctl;
	ListPtr		lst;
	UInt8		i;

	OpenSerial(frm);	
		
	SysCopyStringResource(gSampleLabel, SearchString);
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SampleButton));
	CtlSetLabel(ctl, gSampleLabel);

#ifndef DEMO	
	res = SearchROM(gTimeout);
#else
	rom_list.max = 3;
	SysTaskDelay(sysTicksPerSecond);
#endif	
	CloseSerial(frm);

	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SensorList));
	LstSetListChoices(lst, gSensors, rom_list.max);
	
	return res;
}

static void MainViewInit(void) {
	FormPtr	frm=FrmGetActiveForm();
	Char	String[3];
	UInt16	objectIndex;
	Char*	label;
	TablePtr	tableP;
	UInt16	numRows;
	UInt16	i;
	Char 	tempPtr[10];
	FieldPtr	fldP;

	ListPtr		lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, PeriodList));
	ControlPtr	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, PeriodPopUpTrigger));
	
	LstSetSelection(lst, gAppPrefs.SamplePeriod);
	label = LstGetSelectionText(lst, gAppPrefs.SamplePeriod);
	CtlSetLabel(ctl, label);
	
	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, UnitList));
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, UnitPopUpTrigger));
	
	if (gAppPrefs.Metric) {
		SysCopyStringResource(String, CelsString);
		LstSetSelection(lst, 0);
		label = LstGetSelectionText(lst, 0);
	} else {
		SysCopyStringResource(String, FahrString);
		LstSetSelection(lst, 1);
		label = LstGetSelectionText(lst, 1);
	}
	
	CtlSetLabel(ctl, label);
	
	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SensorList));
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SensorTrigger));
	LstSetSelection(lst, 0);
	label = LstGetSelectionText(lst, 0);
	CtlSetLabel(ctl, label);
	
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, LogCheckbox));
	CtlSetValue(ctl, gAppPrefs.Log);
	
	ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, LogAllCheckbox));
	CtlSetValue(ctl, gAppPrefs.LogAll);
	
	objectIndex = FrmGetObjectIndex(frm, UnitLabelMax);
	FrmHideObject(frm, objectIndex);
	
	FrmCopyLabel(frm, UnitLabelMax, String);
	FrmShowObject(frm, objectIndex);

	objectIndex = FrmGetObjectIndex(frm, UnitLabelMin);
	FrmHideObject(frm, objectIndex);
	
	FrmCopyLabel(frm, UnitLabelMin, String);
	FrmShowObject(frm, objectIndex);

	objectIndex = FrmGetObjectIndex(frm, UnitLabelTemp);
	FrmHideObject(frm, objectIndex);
	
	FrmCopyLabel(frm, UnitLabelTemp, String);
	FrmShowObject(frm, objectIndex);
	
	tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempTable));
	numRows = TblGetNumberOfRows(tableP);

	for (i = 0; i < numRows; i++) {
		TblSetItemStyle(tableP, i, 0, numericTableItem);
		TblSetItemInt(tableP, i, 0, GetRowNum(i));
		
		TblSetItemStyle(tableP, i, 1, textTableItem);
		TblSetItemInt(tableP, i, 1, 10);

		TblSetItemStyle(tableP, i, 2, textTableItem);
		
		TblSetItemStyle(tableP, i, 3, textTableItem);
		TblSetRowSelectable(tableP, i, false);
	}
	
	for (i = 0; i < kNumColumns; i++)
		TblSetColumnUsable(tableP, i, true);

	TblSetLoadDataProcedure(tableP, 1, CustomLoadTemp);
	TblSetLoadDataProcedure(tableP, 3, CustomLoadTemp);
	TblSetLoadDataProcedure(tableP, 2, CustomLoadTemp);
	
	FrmDrawForm(frm);

	if (gAppPrefs.MinTemp < 12500) {
		IntToTemp(gAppPrefs.MinTemp, tempPtr, ",");
		fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMinField));
		FldSetTextPtr(fldP, tempPtr);
		FldDrawField(fldP);

		IntToTemp(gAppPrefs.MaxTemp, tempPtr, ",");
		fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMaxField));
		FldSetTextPtr(fldP, tempPtr);
		FldDrawField(fldP);
	}
	
	UpdateScrollbar();
}

static void Sample(FormPtr frm) {
	Err	err;
	Int16	tempPrec;
	UInt16	i;
	UInt16	j;
	UInt16	min;
	UInt16	max;
	Char	tempPtr[10];
	UInt8	scratch[9];
	FieldPtr	fldP;
	TablePtr	tableP;
	UInt16	recordIndex;
	struct mySampleType	sample;
	struct mySampleType	*samplePtr;
	MemHandle 	myRecordHandle;
	UInt8*		myroms;
	UInt32		temp;

	if (rom_list.max < gSensor + 1) {
		FrmAlert(NoDeviceAlert);
		return;
	}

	if (!OpenSerial(frm)) {
		return;
	}
	
	if (gAppPrefs.LogAll && gAppPrefs.Log) {
		min = 0;
		max = rom_list.max - 1;
	} else {
		min = gSensor;
		max = gSensor;
	}
	
	for (j = min; j <= max; j++) {
		
		do {
#ifndef DEMO
			if(MatchROM(gTimeout, j) < 0) {
				CloseSerial(frm);
				return;
			}
			if(ReadTemperature(gTimeout, gReadTime) < 0) {
				CloseSerial(frm);
				return;
			}
			if(MatchROM(gTimeout, j) < 0) {
				CloseSerial(frm);
				return;
			}
			if(ReadScratchpad(gTimeout, scratch) < 0) {
				CloseSerial(frm);
				return;
			}
#else
			SysTaskDelay(sysTicksPerSecond);
#endif

		} while (scratch[0] == 0xaa && scratch[1] == 0x00 && scratch[6] == 0x0c && scratch[7] == 0x10);
#ifndef DEMO

		myroms = MemHandleLock(rom_list.roms);
		
		if ((myroms[j*8] == DS18B20_FAMILY) || (myroms[j*8] == DS1822_FAMILY)) {
			temp = (scratch[1] << 8) | scratch[0];
			tempPrec = (temp * 100) / 16;
		} else {
			tempPrec = do_temp(scratch);
		}
		
		MemHandleUnlock(rom_list.roms);
#else
		tempPrec = SysRandom(TimGetTicks()) / (sysRandomMax/750);
		tempPrec -= 1800;
#endif	

		if (j == gSensor) {
			if ((tempPrec < gAppPrefs.MinTemp) && (tempPrec > -6000))
				gAppPrefs.MinTemp = tempPrec;
			
			if ((tempPrec > gAppPrefs.MaxTemp) && (tempPrec < 13000))
				gAppPrefs.MaxTemp = tempPrec;

			IntToTemp(tempPrec, gTempString, ",");
			fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempField));
			FldSetTextPtr(fldP, gTempString);
			FldDrawField(fldP);	

			IntToTemp(gAppPrefs.MinTemp, gMinString, ",");
			fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMinField));
			FldSetTextPtr(fldP, gMinString);
			FldDrawField(fldP);

			IntToTemp(gAppPrefs.MaxTemp, gMaxString, ",");
			fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMaxField));
			FldSetTextPtr(fldP, gMaxString);
			FldDrawField(fldP);
		}
	
		if (gAppPrefs.Log) {
			sample.Ticks = TimGetSeconds();
			sample.Temp = tempPrec;
			sample.Sensor = j;
			recordIndex = 0;
			myRecordHandle = DmNewRecord(gTempDB, &recordIndex, sizeof(sample));
			if (DmGetLastErr()) {
				FrmAlert(OutOfMemoryAlert);
				return;
			}
			
			samplePtr = MemHandleLock(myRecordHandle);
			DmWrite(samplePtr, 0, &sample, sizeof(sample));
			MemHandleUnlock(myRecordHandle);
		
			DmReleaseRecord(gTempDB, 0, true);
		
			UpdateScrollbar();

			tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempTable));
		
			for (i = 0; i < kNumRows; i++)
				TblSetItemInt(tableP, i, 0, GetRowNum(i));
		
			TblMarkTableInvalid(tableP);
			TblRedrawTable(tableP);
		}
	}
	
	CloseSerial(frm);
}

static void TransferTable(void) {
	DmOpenRef	MemoDB;
	UInt16	recordIndex;
	MemHandle	myRecordHandle;
	MemPtr		myRecord;
	struct		mySampleType *samplePtr;
	Char*		theMemo;
	Char		tempPtr[12];
	Char		string[30];
	UInt16	i;
	UInt16	size;
	Boolean chop = false;

	SysCopyStringResource(string, TitleString);
	SecondsToString(TimGetSeconds(), tempPtr);
	StrCat(string, tempPtr);
	if (gAppPrefs.Metric)
		SysCopyStringResource(tempPtr, CelsString);
	else
		SysCopyStringResource(tempPtr, FahrString);
	StrCat(string, "(");
	StrCat(string, tempPtr);
	StrCat(string, ")");
	StrCat(string, "\n");
	
	size = StrLen(string);

	for (i = 0; i < DmNumRecords(gTempDB); i++) {
		myRecord = DmQueryRecord(gTempDB, i);
		samplePtr = MemHandleLock(myRecord);
		IntToTemp(samplePtr->Temp, tempPtr, ".");
		
		size += StrLen(tempPtr) + 1;
		size += 11;

		if ((size >= 65450) && (chop == false)) {
			if (FrmAlert(RecordTooLongAlert) == 1)
				return;
			chop = true;
			break;
		}
		
		MemHandleUnlock(myRecord);
	}
	size++;
	
	MemoDB = DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
	recordIndex = DmNumRecords(MemoDB);
	myRecordHandle = DmNewRecord(MemoDB, &recordIndex, size);
	
	if (DmGetLastErr()) {
		FrmAlert(OutOfMemoryAlert);
		DmCloseDatabase(MemoDB);
		return;
	}

	
	theMemo = MemHandleLock(myRecordHandle);
	
	size = 0;
	
	DmStrCopy(theMemo, size, string);
	
	size = StrLen(string);
		
	for (i = 0; i < DmNumRecords(gTempDB); i++) {
		myRecord = DmQueryRecord(gTempDB, i);
		samplePtr = MemHandleLock(myRecord);
		
		SecondsToString(samplePtr->Ticks, tempPtr);
		DmStrCopy(theMemo, size, tempPtr);
		DmStrCopy(theMemo, size+StrLen(tempPtr), "\t");
		size += StrLen(tempPtr) + 1;
		
		StrIToA(tempPtr, samplePtr->Sensor + 1);
		DmStrCopy(theMemo, size, tempPtr);
		size+=StrLen(tempPtr);
		DmStrCopy(theMemo, size, "\t");
		size++;

		IntToTemp(samplePtr->Temp, tempPtr, ".");
		DmStrCopy(theMemo, size, tempPtr);
		DmStrCopy(theMemo, size + StrLen(tempPtr), "\n");
		size += StrLen(tempPtr) + 1;
		
		MemHandleUnlock(myRecord);
		
		if (size > 65450)
			break;
	}
	
	MemHandleUnlock(myRecordHandle);
	
	FrmCustomAlert(TransferAlert, StrIToA(tempPtr, DmNumRecords(gTempDB)), NULL, NULL);
	
	DmReleaseRecord(MemoDB, recordIndex, true);
	DmCloseDatabase(MemoDB);
}

static Boolean MainViewHandleEvent(EventPtr event) {
	Boolean	handled = false;
	Char	tempPtr[10];
	FormPtr	frm = FrmGetActiveForm();
	FieldPtr	fldP;
	TablePtr	tableP;
	UInt32	ticks;
	UInt16	i;
	UInt16	textHeight;
	Char	String[3];
	ControlPtr	ctl;
	UInt16	objectIndex;
	UInt8	perc;
	
	switch (event->eType) {
	case frmOpenEvent:
		MainViewInit();

		SysBatteryInfo(false, NULL, NULL, NULL, NULL, NULL, &perc);
		StrIToA(tempPtr, perc);
		StrCat(tempPtr, " %");
		objectIndex = FrmGetObjectIndex(frm, BattLabel);
		FrmHideObject(frm, objectIndex);
		FrmCopyLabel(frm, BattLabel, tempPtr);
		FrmShowObject(frm, objectIndex);

		if (GetROM(frm) < 0) {

			handled = true;
			break;
		}

		Sample(frm);

		gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;
		handled = true;
		break;
	case menuEvent:
		switch(event->data.menu.itemID) {
		case About:
#ifndef DS1820
			FrmAlert(AboutBoxAlert);
			handled = true;
#else
			FrmAlert(AboutBox1820Alert);
			handled = true;
#endif
		}
		break;
	case sclExitEvent:
		gOffset = event->data.sclExit.newValue;

		tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempTable));
		
		for (i = 0; i < kNumRows; i++)
			TblSetItemInt(tableP, i, 0, GetRowNum(i));
		
		TblMarkTableInvalid(tableP);
		TblRedrawTable(tableP);

		break;
	case popSelectEvent:
		switch (event->data.popSelect.controlID) {
		case UnitPopUpTrigger:
			if (event->data.popSelect.selection == 0) {
				gAppPrefs.Metric = true;		
				SysCopyStringResource(String, CelsString);
	
			} else {
				gAppPrefs.Metric = false;
				SysCopyStringResource(String, FahrString);
			}
			
			if (event->data.popSelect.selection != event->data.popSelect.priorSelection) {
				Sample(frm);
				gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;

				objectIndex = FrmGetObjectIndex(frm, UnitLabelMax);
				FrmHideObject(frm, objectIndex);

				FrmCopyLabel(frm, UnitLabelMax, String);
				FrmShowObject(frm, objectIndex);

				objectIndex = FrmGetObjectIndex(frm, UnitLabelMin);
				FrmHideObject(frm, objectIndex);

				FrmCopyLabel(frm, UnitLabelMin, String);
				FrmShowObject(frm, objectIndex);

				objectIndex = FrmGetObjectIndex(frm, UnitLabelTemp);
				FrmHideObject(frm, objectIndex);

				FrmCopyLabel(frm, UnitLabelTemp, String);
				FrmShowObject(frm, objectIndex);

			}
			break;
		case PeriodPopUpTrigger:
			gAppPrefs.SamplePeriod = event->data.popSelect.selection;
			gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;
			break;
		case SensorTrigger:
			gSensor = event->data.popSelect.selection;

			break;
		}
		break;
		
	case keyDownEvent:
		if (TxtCharIsHardKey(event->data.keyDown.modifiers, event->data.keyDown.chr)) {
			Sample(frm);
			gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;
			SysBatteryInfo(false, NULL, NULL, NULL, NULL, NULL, &perc);
			StrIToA(tempPtr, perc);
			StrCat(tempPtr, " %");
			objectIndex = FrmGetObjectIndex(frm, BattLabel);
			FrmHideObject(frm, objectIndex);
			FrmCopyLabel(frm, BattLabel, tempPtr);
			FrmShowObject(frm, objectIndex);

			handled = true;
			break;

		}
		if (DmNumRecords(gTempDB) < kNumRows) {
			handled = true;
			break;
		}

		if (event->data.keyDown.chr == pageUpChr) {
				
			if (gOffset >= kNumRows - 1) {
				gOffset -= (kNumRows - 1);
			} else if (gOffset > 0) {
				gOffset = 0;
			}
			handled = true;
		} else if (event->data.keyDown.chr == pageDownChr) {
			if ((gOffset <= (DmNumRecords(gTempDB) - 2 * kNumRows + 1)) && DmNumRecords(gTempDB) > 2 * kNumRows) {
				gOffset += (kNumRows - 1);
			} else if (gOffset < DmNumRecords(gTempDB) - kNumRows) {
				gOffset = DmNumRecords(gTempDB) - kNumRows;
			}
		}
		UpdateScrollbar();
		tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempTable));
		
		for (i = 0; i < kNumRows; i++)
			TblSetItemInt(tableP, i, 0, GetRowNum(i));
		
		TblMarkTableInvalid(tableP);
		TblRedrawTable(tableP);
		handled = true;
		
		break;
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		case ResetButton:
			gAppPrefs.MinTemp = 13000;
			gAppPrefs.MaxTemp = -6000;
		

			StrCopy(tempPtr,"");
			fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMinField));
			FldSetTextPtr(fldP, tempPtr);
			FldDrawField(fldP);

			StrCopy(tempPtr, "");
			fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempMaxField));
			FldSetTextPtr(fldP, tempPtr);
			FldDrawField(fldP);
			handled = true;
			break;

		case SampleButton:
			Sample(frm);
			gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;
			SysBatteryInfo(false, NULL, NULL, NULL, NULL, NULL, &perc);
			StrIToA(tempPtr, perc);
			StrCat(tempPtr, " %");
			objectIndex = FrmGetObjectIndex(frm, BattLabel);
			FrmHideObject(frm, objectIndex);
			FrmCopyLabel(frm, BattLabel, tempPtr);
			FrmShowObject(frm, objectIndex);

			handled = true;
			break;
		case LogCheckbox:
			if (event->data.ctlSelect.on) {
				gAppPrefs.Log = true;
			 } else {
				gAppPrefs.Log = false;
			}
				
			handled = true;
			break;
		
		case LogAllCheckbox:
			if (event->data.ctlSelect.on) {
				gAppPrefs.LogAll = true;
			} else {
				gAppPrefs.LogAll = false;
			}
			
			handled = true;
			break;
		
		case ListClearButton:
			
			textHeight = DmNumRecords(gTempDB);
			
			for (i = 0; i < textHeight; i++) {
				DmRemoveRecord(gTempDB, 0);
			}
			
			gOffset = 0;
			UpdateScrollbar();

			tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TempTable));
		
			for (i = 0; i < kNumRows; i++)
				TblSetItemInt(tableP, i, 0, GetRowNum(i));
		
			TblMarkTableInvalid(tableP);
			TblRedrawTable(tableP);

			handled = true;
			break;
		case TransferButton:
			TransferTable();
			handled = true;
			break;
		case SrchROMButton:
			if (GetROM(frm) < 0) {
				handled = true;
				break;
			}
			handled = true;
			break;
		}
		break;
				
	case nilEvent: 
		ticks = TimGetTicks();
		if (gSamplePeriods[gAppPrefs.SamplePeriod])
			EvtResetAutoOffTimer();
			
		SysBatteryInfo(false, NULL, NULL, NULL, NULL, NULL, &perc);
		StrIToA(tempPtr, perc);
		StrCat(tempPtr, " %");
		objectIndex = FrmGetObjectIndex(frm, BattLabel);
		FrmHideObject(frm, objectIndex);
		FrmCopyLabel(frm, BattLabel, tempPtr);
		FrmShowObject(frm, objectIndex);

		if ((ticks > gNextSample) && (gAppPrefs.SamplePeriod != 0)) {


		
			Sample(frm);
			gNextSample = TimGetTicks() + gSamplePeriods[gAppPrefs.SamplePeriod] * sysTicksPerSecond;

			handled = true;
		}
		break;
	}
	return handled;
}


static Boolean ApplicationHandleEvent(EventPtr event) {
	FormPtr	frm;
	UInt16	formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent) {
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);
		
		switch (formId) {
		case TempMainForm:
			FrmSetEventHandler(frm, MainViewHandleEvent);
			break;
		
		}
		handled = true;
	}
	return handled;
}

static UInt32 NextSample() {
	UInt32	nextSample;
	UInt32	autoOff;
	UInt32	ticks;
	Char	winPtr[10];
	
	autoOff = SysSetAutoOffTime(0);
	SysSetAutoOffTime(autoOff);
	
	ticks = TimGetTicks();
	
	if (gNextSample >= ticks)
		nextSample = gNextSample - ticks;
	else
		nextSample = 0;
		
	autoOff *= sysTicksPerSecond;
	
	if ((autoOff != 0 ) && (autoOff < nextSample))
		nextSample = autoOff - 1;

	if (gAppPrefs.SamplePeriod == 0)
		nextSample = evtWaitForever;
	else
		EvtResetAutoOffTimer();

	return nextSample;
}

static void EventLoop(void) {
	EventType	event;
	UInt16		error;
	
	do {
		EvtGetEvent(&event, NextSample());
		if(!SysHandleEvent(&event))
			if(!MenuHandleEvent(0, &event, &error))
				if (!ApplicationHandleEvent(&event))
					FrmDispatchEvent(&event);
	}
	while (event.eType != appStopEvent);
}

UInt32	PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
	
	Err err;
	
/*	err = RomVersionCompatible(version20, launchFlags);
	if (err)
		return err;
*/	
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		if (!StartApplication(launchFlags)) {
			FrmGotoForm(TempMainForm);
			EventLoop();
			StopApplication();
		}
	}
	return err;
}
