/*
  myTemp is used to connect PalmOS PDAs to Dallas Microlan(tm) Devices
  It currently supports DS1820/DS18S20 Digital Thermometers
  Copyright (C) 2000 Christof Klaiber

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA  02111-1307  USA


  Christof Klaiber <christof.klaiber@gmx.de>
  Lachnerstr. 22
  76131 Karlsruhe
  Germany
*/

#include <PalmOS.h>
#include <SerialMgrOld.h>

#include "onewire.h"

/*
	Name:		TouchReset
	Purpose:	Reset the 1-wire Bus
	Input:		UInt16	timeout	Timeout-Value in Ticks
	Globals:	UInt16	gSerialRefNum	Reference to opened Serial Port
			
	Result:		UInt8		0: no presence detected
					1: presence pulse no alarm
					-1: serial error
	Comments: "2: alarm followed by presence pulse" is not implemented
	 
	 Name: Christof Klaiber <christof.klaiber@gmx.de>
	 Versions:	25.01.2001	First working Release
	 
	 This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
	
*/

Int8 TouchReset(UInt16 timeout) {
	UInt8	wbuff, stat;
	
	SerSettingsType	term;
	Err	err;
	
	
	/* Flush Send and Receive Queue */
	SerReceiveFlush(gSerialRefNum, 1);
	SerSendFlush(gSerialRefNum);
	
	/* Change Serial Settings to 9600 Baud */
	/* Reset Pulse needs to be that long */
	term.baudRate = 9600;
	term.flags = serDefaultSettings;
	SerSetSettings(gSerialRefNum, &term);
	
	/* Send the Reset Pulse */
	wbuff = 0xf0;
	err = SerClearErr(gSerialRefNum);
	SerSend(gSerialRefNum, &wbuff, 1, &err);
	
	/* Wait for answer */
	err = SerReceiveWait(gSerialRefNum, 1, timeout);
	
	if (err)
		return -1;
	
	/* Get the answer, if any */
	SerReceive(gSerialRefNum, &wbuff, 1, 1, &err);
	if (wbuff != 0xf0)
		/* 1-wire Device detected */
		stat = 1;
	else
		/* Nothing detected */
		stat = 0;
	
	/* reinitialize the Port */
	term.baudRate = 115200;
	term.flags = serDefaultSettings;
	SerSetSettings(gSerialRefNum, &term);
	
	return stat;
}

/*
	Name:		TouchBits
	Purpose:	Send/Receive Bits to/from 1-wire Devices
	Input:		UInt16	timeout	Timeout-Value in Ticks
			UInt8	nbits	# of Bits to send
			UInt8	outch	Value for Output
	Globals:	UInt16	gSerialRefNum	Reference to opened Serial Port
			
	Result:		Int16	Received Byte from Bus -1 if error
			
	 Name: Christof Klaiber <christof.klaiber@gmx.de>
	 Versions:	25.01.2001	First working Release
	 
	 This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
	
*/


Int16 TouchBits(UInt16 timeout, UInt8 nbits, UInt8 outch) {
	UInt8	sendbit, Mask, inch, wbuff;
	UInt16	x;
	Err	err;
	
	/* Flush Input and Output Queue */
	SerReceiveFlush(gSerialRefNum, 1);
	SerSendFlush(gSerialRefNum);
	
	/* Initialize the Mask to get the Bits one by one */
	Mask = 1;
	/* Result of this Function */
	inch = 0;
	
	/* Prepare First Bit to output */
	sendbit = (outch & 0x1) ? 0xff: 0x00;
	
	/* Iterrate through the Bits to send */
	for (x = 0; x < nbits; x++) {
		/* Clear pending errors from the serial device */
		err = SerClearErr(gSerialRefNum);
		/* Send the Bit */
		SerSend(gSerialRefNum, &sendbit, 1, &err);
		Mask <<= 1;
		sendbit = (outch & Mask)?0xff: 0x00;
		
		inch >>= 1;
		
		/* Get Output from the Queue if there is one */
		for(;;) {
			SerReceive(gSerialRefNum, &wbuff, 1, timeout, &err);
			if (err != serErrTimeOut) {
				inch |= (wbuff & 0x01)? 0x80:0x00;
				break;
			} else
				return 0xff;
		}
	}
	return inch;
}

/*
	Name:		TouchByte
	Purpose:	Wrapper for TouchBits, Send/Receive a Byte
	Input:		UInt16	timeout	Timeout-Value in Ticks
			UInt8	outch	Value for Output
	Globals:	UInt16	gSerialRefNum	Reference to opened Serial Port
			
	Result:		Int16	Received Byte from Bus -1 if error
			
	 Name: Christof Klaiber <christof.klaiber@gmx.de>
	 Versions:	25.01.2001	First working Release
	 
	 This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
	
*/


Int16 TouchByte(UInt16 timeout, UInt8 outch)
{
	return TouchBits(timeout, 8, outch);
}
